package jp.co.sds_corp.tool.check;

import java.applet.Applet;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.Proxy;
import java.net.ProxySelector;
import java.net.URLConnection;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

public class NetworkChecker extends Checker
{
	ResourceBundle prop = ResourceBundle.getBundle("check");
	Properties remoteProp = new Properties();
	Applet applet;
	BlockingQueue<Result> queue = new LinkedBlockingQueue<Result>();
	int count;
	int timeout = 10;
	int retryCount;
	List<Integer> listRetryCode = new ArrayList<Integer>();

	protected NetworkChecker(Applet applet) {
		this.applet = applet;
	}

	@Override
	public int getCheckCount() {
		return 10;
	}

	@Override
	public boolean check() {
		fireProgressNotify(ProgressListener.Type.HOLD, "ネットワークの確認");
		getRemoteProperties();
		count = 0;
		clearError();
		String err = null;
		boolean ret = true;
		String var = getProperty("url.count");
		int count = Integer.valueOf(var).intValue();
		for (int i = 0; i < count; i++) {
			if (!checkUrl(getProperty("url." + (i+1)))) {
				ret = false;
			}
			if (err == null) {
				err = getError();
			} else {
				err = err + "\n" + getError();
			}
		}
		setError(err);
		return ret;
	}

	String getProperty(String key) {
		String ret;
		ret = remoteProp.getProperty(key);
		if (ret == null) {
			ret = prop.getString(key);
		}
		return ret;
	}

	void getRemoteProperties() {
		remoteProp.clear();
		try {
			URL url = new URL(applet.getCodeBase() + "check.properties");
			URLConnection conn = url.openConnection();
			conn.setDoInput(true);
			conn.setUseCaches(false);
			conn.connect();

			remoteProp.load(conn.getInputStream());
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	boolean checkUrl(String surl) {
		boolean ret = true;
		URL url = null;
		try {
			url = new URL(surl);
		} catch (MalformedURLException ex) {
			String msg = Message.getString("90302");
			setError(msg);
			addDescription(ex);
			return false;
		}
		checkProxy(url);
		for (int i = 0; i < 5; i++) {
			count++;
			fireProgressNotify(ProgressListener.Type.FORWARD,
					"ネットワークの確認 (" + count + "/" + getCheckCount() + ")");
			if (!request(url)) {
				ret = false;
			}
		}
		return ret;
	}

	void checkProxy(URL url) {
		try {
			ProxySelector selector = ProxySelector.getDefault();
			List<Proxy> list = selector.select(url.toURI());
			for (Proxy proxy : list) {
				addInfo("url=" + url + " proxy=" + proxy);
			}
			if (list.size() == 0) {
				addInfo("url=" + url + " proxy=DIRECT");
			}
		} catch (Exception ex) {
			addDescription("select proxy error");
			addDescription(ex);
		}
	}

	boolean request(URL url) {
		boolean ret = true;
		Requester requester = new Requester(url);
		Thread thr = new Thread(requester);
		thr.start();
		try {
			Result res = waitResult(url);
		} catch (UnknownHostException ex) {
			ret = false;
			String msg = Message.formatMessage("90301", url.getHost());
			setError(msg);
			addDescription(ex);
		} catch (Exception ex) {
			ret = false;
			String msg = Message.formatMessage("90302", url.getHost());
			setError(msg);
			addDescription(ex);
		}
		return ret;
	}

	Result waitResult(URL url) throws Exception {
		Result res = queue.poll(timeout, TimeUnit.SECONDS);
		if (res == null) {
			throw(new TimeoutException("timeout [" + url + "]"));
		}
		if (res.ex != null) {
			throw(res.ex);
		}
		return res;
	}

	class Result
	{
		int responseCode;
		Exception ex;
	}

	class Requester implements Runnable
	{
		URL url;
		Requester(URL url) {
			this.url = url;
		}
		public void run() {
			Result res = new Result();
			Date start = new Date();
			boolean usingProxy = false;
			boolean connected = false;
			HttpURLConnection conn = null;
			try {
				conn = (HttpURLConnection) url.openConnection();
				conn.setDoInput(true);
				conn.setDoOutput(true);
				conn.setUseCaches(false);
				conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
				conn.setRequestMethod("POST");
				conn.connect();

				connected = true;
				usingProxy = conn.usingProxy();

				int responseCode = conn.getResponseCode();
				res.responseCode = responseCode;
				if (responseCode == HttpURLConnection.HTTP_OK ||
					responseCode == HttpURLConnection.HTTP_BAD_REQUEST ||
					responseCode == HttpURLConnection.HTTP_FORBIDDEN)
				{
					StringBuilder result = new StringBuilder();
					BufferedReader rd = new BufferedReader(
						new InputStreamReader(conn.getInputStream()));
					for ( ;; ) {
						String line = rd.readLine();
						if (line == null) {
							break;
						}
						result.append(line);
//						System.out.println(line);
					}
					rd.close();
				} else {
					String msg = "response error url=" + url;
					throw(new RequestException(msg, responseCode,
						conn.getResponseMessage()));
				}
				Date end = new Date();
				long ms = end.getTime() - start.getTime();
				addInfo("url=" + url + " proxy=" + usingProxy + " ms=" + ms
					+ " response=" + responseCode
					+ ":" + conn.getResponseMessage());
				queue.put(res);
			} catch (Exception ex) {
				boolean up = false;
				if (connected) {
					up = usingProxy;
				} else {
					up = conn.usingProxy();
				}
				Date end = new Date();
				long ms = end.getTime() - start.getTime();
				String info = "url=" + url + " proxy=" + up + " ms=" + ms;
				try {
					info += " response=" + conn.getResponseCode()
						+ ":" + conn.getResponseMessage();
				} catch (Exception ioex) {
					info += "getResponse error:" + ioex;
				}
				addInfo(info);
				try {
					res.ex = ex;
					queue.put(res);
				} catch (InterruptedException iex) {
				}
			} finally {
				if (conn != null) {
					try {
						conn.disconnect();
					} catch (Exception ex) {
					}
				}
			}
		}
	}

}
