package	jp.co.sds_corp.tool.log;

import	java.io.BufferedReader;
import	java.io.FileInputStream;
import	java.io.InputStream;
import	java.io.InputStreamReader;
import	java.text.MessageFormat;
import	java.text.ParseException;
import	java.util.ArrayList;
import	java.util.List;

import	jp.co.sds_corp.tool.Where;
import jp.co.sds_corp.util.log.CmnLogFactory;
import	jp.co.sds_corp.util.log.CmnLogger;

public class AccessLogViewer
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	String where;
	List<Where> listWhere = new ArrayList<Where>();
	MessageFormat format = new MessageFormat("{0} {1} {2} [{3}] \"{4}\" {5} {6} {7} {8}");
	static class Record {
		String line;
		String host;
		String log;
		String user;
		String date;
		String request;
		String status;
		String bytes;
		String time;
		String connection;
		public String toString() {
			return line;
		}
	}

	boolean checkOption( String param ) throws ParseException {
		if ( param.startsWith( "-where:" )) {
			where = param.substring( 7 );
			parseWhere( where );
			return true;
		}
		return false;
	}

	void parseWhere(String param) throws ParseException {
		String[] arr = param.split(" ");
		for (String var : arr) {
			Where where = new Where(var);
			listWhere.add(where);
		}
	}

	void printBuffer(Record record) throws Exception {

		if (!matchWhere(record)) {
			return;
		}

		System.out.println( record.toString() );
	}

	boolean matchWhere(Record record) {
		for (Where where : listWhere) {
			if (!matchWhere(record, where)) {
				return false;
			}
		}
		return true;
	}

	boolean matchWhere(Record record, Where where) {
		String field = where.getKey();
		String value = null;
		if (field.equals("host")) {
			value = record.host;
		} else
		if (field.equals("log")) {
			value = record.log;
		} else
		if (field.equals("user")) {
			value = record.user;
		} else
		if (field.equals("date")) {
			value = record.date;
		} else
		if (field.equals("request")) {
			value = record.request;
		} else
		if (field.equals("status")) {
			value = record.status;
		} else
		if (field.equals("bytes")) {
			value = record.bytes;
		} else
		if (field.equals("time")) {
			value = record.time;
		} else
		if (field.equals("connection")) {
			value = record.connection;
		}
		boolean ret = where.match(value);
		return ret;
	}

	Record parseLine(String line) {
		Record ret = new Record();
		ret.line = line;
		try {
			Object[] arr = format.parse(line);
/*
			for ( int i = 0; i < arr.length; i++ ) {
				System.out.println( arr[i] );
			}
*/
			if (arr.length != 9) {
				return null;
			}
			ret.host = (String )arr[0];
			ret.log = (String )arr[1];
			ret.user = (String )arr[2];
			ret.date = (String )arr[3];
			ret.request = (String )arr[4];
			ret.status = (String )arr[5];
			ret.bytes = (String )arr[6];
			ret.time = (String )arr[7];
			ret.connection = (String )arr[8];
		} catch (Exception ex) {
			return null;
		}
		return ret;
	}

	public void start( InputStream is ) throws Exception {
		BufferedReader br = new BufferedReader(
			new InputStreamReader( is ));
		for ( ;; ) {
			String line = br.readLine();
			if ( line == null ) {
				break;
			}
			Record record = parseLine(line);
			if (record != null) {
				printBuffer(record);
			}
		}
		br.close();
	}

	public static void main( String[] args ) {
		try {
			AccessLogViewer obj = new AccessLogViewer();
			int count = 0;
			for ( int i = 0; i < args.length; i++ ) {
				String file = args[i];
				if ( obj.checkOption( file )) {
					continue;
				}
				System.out.println( "open " + file );
				obj.start( new FileInputStream( file ));
				count++;
			}
			if ( count == 0 ) {
				obj.start( System.in );
			}
		} catch ( Exception ex ) {
			logger.error( AccessLogViewer.class, "error", ex );
		}
	}
}
