package	jp.co.sds_corp.tool.log;

import	java.io.BufferedReader;
import	java.io.ByteArrayInputStream;
import	java.io.FileInputStream;
import	java.io.InputStream;
import	java.io.InputStreamReader;
import	java.text.SimpleDateFormat;
import	java.util.Date;
import	java.util.Locale;

import  javax.xml.parsers.DocumentBuilder;
import  javax.xml.parsers.DocumentBuilderFactory;

import  org.w3c.dom.Document;
import  org.w3c.dom.Element;
import  org.w3c.dom.Node;
import  org.w3c.dom.NodeList;

import	jp.co.sds_corp.util.config.NullResolver;
import jp.co.sds_corp.util.log.CmnLogFactory;
import	jp.co.sds_corp.util.log.CmnLogger;

public class GcViewer
{
	public static final String DEFER_NODE_EXPANSION
		= "http://apache.org/xml/features/dom/defer-node-expansion";
	static final CmnLogger logger = CmnLogFactory.getLogger();
	String file;
	SimpleDateFormat sdfPrm = new SimpleDateFormat( "yyyyMMddHHmmss" );
	SimpleDateFormat sdf = new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss" );
	SimpleDateFormat sdfTs = new SimpleDateFormat( "MMM dd HH:mm:ss yyyy", Locale.US );
	long rangeFr = -1;
	long rangeTo = -1;
	DocumentBuilder builder;

	enum GcType {
		SYS("sys"),
		AF("af"),
		NURSERY(null,"nursery"),
		TENURED(null,"tenured"),
		CONCURRENT("con"),
		NONE(null);

		String tag;
		String type;
		GcType(String tag) {
			this.tag = tag;
		}
		GcType(String tag, String type) {
			this(tag);
			this.type = type;
		}
		String getTag() {
			return this.tag;
		}
		String getType() {
			return this.type;
		}
	};
	static class MemoryInfo {
		long freeBytes;
		long totalBytes;
		int percent;
	}
	static class GcInfo {
		GcType type;
		Date timestamp;
		String gcType;
		double gcMs;
		double totalMs;
		MemoryInfo nurseryBefore;
		MemoryInfo tenuredBefore;
		MemoryInfo nurseryAfter;
		MemoryInfo tenuredAfter;
	}

	public void init() throws Exception {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
//		factory.setFeature( ConfigurationManager.DEFER_NODE_EXPANSION, false );
		builder = factory.newDocumentBuilder();
		builder.setEntityResolver( new NullResolver() );
	}

	public void start() throws Exception {
		logger.info( this, "open " + file );
		start( new FileInputStream( file ));
	}

	void printHeader() {
		System.out.println( "timestamp\ttype\tgcType\tgcMs\ttotalMs\t"
			+ "nursery(before) free\ttotal(NB)\tpercent(NB)\t"
			+ "tenured(before) free\ttotal(TB)\tpercent(TB)\t"
			+ "nursery(after) free\ttotal(NA)\tpercent(NA)\t"
			+ "tenured(after) free\ttotal(TA)\tpercent(TA)\t"
			);
	}

	void printInfo(GcInfo info) throws Exception {
		if (info == null) {
			return;
		}
		long time = info.timestamp.getTime();
		if ((rangeFr >= 0 && rangeFr > time) ||
			(rangeTo >= 0 && rangeTo < time))
		{
			return;
		}
		String msg = sdf.format(info.timestamp) + "\t"
			+ info.type + "\t"
			+ info.gcType + "\t"
			+ info.gcMs + "\t"
			+ info.totalMs + "\t";
		if (info.nurseryBefore == null) {
			msg += "\t\t\t";
		} else {
			msg += info.nurseryBefore.freeBytes + "\t"
			+ info.nurseryBefore.totalBytes + "\t"
			+ info.nurseryBefore.percent + "\t";
		}
		if (info.tenuredBefore == null) {
			msg += "\t\t\t";
		} else {
			msg += info.tenuredBefore.freeBytes + "\t"
			+ info.tenuredBefore.totalBytes + "\t"
			+ info.tenuredBefore.percent + "\t";
		}
		if (info.nurseryAfter == null) {
			msg += "\t\t\t";
		} else {
			msg += info.nurseryAfter.freeBytes + "\t"
			+ info.nurseryAfter.totalBytes + "\t"
			+ info.nurseryAfter.percent + "\t";
		}
		if (info.tenuredAfter == null) {
			msg += "\t\t\t";
		} else {
			msg += info.tenuredAfter.freeBytes + "\t"
			+ info.tenuredAfter.totalBytes + "\t"
			+ info.tenuredAfter.percent + "\t";
		}
		System.out.println(msg);
	}

	GcInfo parseXml(GcType type, String buf) throws Exception {
		// logger.trace("buf=" + buf);
		ByteArrayInputStream bis = new ByteArrayInputStream( buf.getBytes());
		Document doc = builder.parse( bis );

		Element root = doc.getDocumentElement();
		if (type == GcType.CONCURRENT) {
			String event = root.getAttribute("event");
			if (event != null && !event.equals("collection")) {
				// ignore "kickoff", "completed sweep"
				return null;
			}
		} else
		if (type == GcType.AF) {
			String attrType = root.getAttribute("type");
			if (attrType != null) {
				for (GcType gt : GcType.values()) {
					if (gt.getType() == null) {
						continue;
					}
					if (attrType.equals(gt.getType())) {
						type = gt;
						break;
					}
				}
			}
		}

		GcInfo info = new GcInfo();
		info.type = type;
		String ts = root.getAttribute("timestamp");
		info.timestamp = sdfTs.parse(ts);
		info.totalMs = getTotalMs(root);

		Element gc = getChildElement(root, "gc");
		if (gc != null) {
			info.gcType = gc.getAttribute("type");
			info.gcMs = getTotalMs(gc);
		}

		info.nurseryBefore = parseMemoryInfo(getChildElement(root, "nursery"));
		info.tenuredBefore = parseMemoryInfo(getChildElement(root, "tenured"));
		info.nurseryAfter = parseMemoryInfo(getChildElement(root, "nursery", 1));
		info.tenuredAfter = parseMemoryInfo(getChildElement(root, "tenured", 1));
		return info;
	}

	MemoryInfo parseMemoryInfo(Element ele) {
		if (ele == null) {
			return null;
		}
		MemoryInfo info = new MemoryInfo();
		info.freeBytes = getLongAttribute(ele, "freebytes");
		info.totalBytes = getLongAttribute(ele, "totalbytes");
		info.percent = (int )getLongAttribute(ele, "percent");
		return info;
	}

	double getTotalMs(Element ele) throws Exception {
		double ret = 0.0;
		Element time;
		time = getChildElement(ele, "time");
		if (time != null) {
			ret = getDoubleAttribute(time, "totalms");
		}
		time = getChildElement(ele, "time", 1);
		if (time != null) {
			ret = getDoubleAttribute(time, "totalms");
		}
		time = getChildElement(ele, "timesms");
		if (time != null) {
			ret = getDoubleAttribute(time, "total");
		}
		return ret;
	}

	double getDoubleAttribute(Element ele, String name) {
		double ret = 0.0;
		String var = ele.getAttribute(name);
		if (var != null && var.length() > 0) {
			ret = Double.valueOf(var).doubleValue();
		}
		return ret;
	}

	long getLongAttribute(Element ele, String name) {
		long ret = 0;
		String var = ele.getAttribute(name);
		if (var != null && var.length() > 0) {
			ret = Long.valueOf(var).longValue();
		}
		return ret;
	}

	Element getChildElement( Element element, String name ) {
		return getChildElement( element, name, 0 );
	}

	Element getChildElement( Element element, String name, int pos ) {
		int found = 0;
		Element ret = null;
		NodeList list = element.getChildNodes();
		for ( int i = 0; i < list.getLength(); i++ ) {
			Node child = list.item( i );
			if ( child.getNodeType() == Node.ELEMENT_NODE &&
					child.getNodeName().equals( name ))
			{
				found++;
				if (found > pos) {
					ret = (Element )child;
					break;
				}
			}
		}
		return ret;
	}

	public void start( InputStream is ) throws Exception {
		StringBuilder buf = new StringBuilder();
		GcType type = GcType.NONE;
		int startLine = -1;
		int lineNo = 0;
		int count = 0;
		BufferedReader br = new BufferedReader( new InputStreamReader( is ));
		printHeader();
		for ( ;; ) {
			String line = br.readLine();
			if ( line == null ) {
				break;
			}
			lineNo++;

			String trim = line.trim();
			for (GcType gt : GcType.values()) {
				if (gt.getTag() == null) {
					continue;
				}
				String tag = "<" + gt.getTag() + " ";
				if (trim.startsWith(tag)) {
					if (type == GcType.NONE) {
						type = gt;
						count = 0;
					}
					count++;
					startLine = lineNo;
					break;
				}
			}
			if (type == GcType.NONE) {
				if (trim.length() > 0) {
					logger.debug(this, "? " + line);
				}
				continue;
			} else {
				buf.append(line);
			}

			String endStr = "</" + type.getTag() + ">";
			if (trim.indexOf(endStr) >= 0) {
				count--;
				if (count == 0) {
					try {
						GcInfo info = parseXml(type, buf.toString());
						printInfo(info);
					} catch (Exception ex) {
						logger.error(this, "parse error line=(" + startLine + ")", ex);
					}
					buf.setLength(0);
					type = GcType.NONE;
				}
			}
		}
		br.close();
	}

	void usage() {
		logger.error(this, "usage: [file] {from(yyyyMMddHHmmss)} {to(yyyyMMddHH:mmss)}" );
	}

	boolean checkArgs( String[] args ) {
		if ( args.length < 1 ) {
			return false;
		}
		file = args[0];
		if ( args.length >= 2 ) {
			try {
				Date dt = sdfPrm.parse(args[1]);
				rangeFr = dt.getTime();
			} catch (Exception ex) {
				logger.error(this, "from parse error" + ex);
				return false;
			}
		}
		if ( args.length >= 3 ) {
			try {
				Date dt = sdfPrm.parse(args[2]);
				rangeTo = dt.getTime();
			} catch (Exception ex) {
				logger.error(this, "from parse error" + ex);
				return false;
			}
		}
		return true;
	}

	public static void main( String[] args ) {
		try {
			GcViewer obj = new GcViewer();
			if (!obj.checkArgs(args)) {
				obj.usage();
				System.exit( 9 );
			}

			obj.init();
			obj.start();
		} catch ( Exception ex ) {
			logger.error( GcViewer.class, "error", ex );
		}
	}
}
