package jp.co.sds_corp.tool.shell;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.jcoverage.coverage.Instrumentation;

public class ShellCoverageParser
{
	Map<String,Instrumentation> instMap = new HashMap<String,Instrumentation>();
	Map<String,List<Integer>> resMap = new HashMap<String,List<Integer>>();
	Pattern linePattern = Pattern.compile("\\++ (.*)\\(([0-9]+)\\) .*");
	File sourceDir = null;

	public ShellCoverageParser(String result) throws Exception {
		parseResult(new File(result));
	}

	void parseResult(File result) throws Exception {
		BufferedReader rd = new BufferedReader(
				new FileReader(result));
		for ( ;; ) {
			String line = rd.readLine();
			if (line == null) {
				break;
			}
			parseLine(line);
		}
		rd.close();
	}

	void parseLine(String line) throws Exception {
		Matcher matcher;
		try {
			matcher = linePattern.matcher(line);
		} catch (Exception ex) {
			return;
		}
		if (!matcher.matches()) {
			return;
		}
		String shell = matcher.group(1);
		String lineNo = matcher.group(2);
//		System.out.println("shell=" + shell + " line=" + lineNo);

		List<Integer> coverages = resMap.get(shell);
		if (coverages == null) {
			coverages = new ArrayList<Integer>();
			resMap.put(shell,  coverages);
		}
		coverages.add(Integer.valueOf(lineNo));
	}

	public void setSourceDir(File sourceDir) {
		this.sourceDir = sourceDir;
	}

	public void generate(File reportDir) throws Exception {
		ShellReportDriver driver = new ShellReportDriver(sourceDir);

		for (String shell : resMap.keySet()) {
			Instrumentation inst = instMap.get(shell);
			if (inst == null) {
				ShellInstrumentBuilder builder = new ShellInstrumentBuilder(shell);
				inst = builder.getInstrumentation();
				if (inst == null) {
					continue;
				}
				instMap.put(shell, inst);
			}
			List<Integer> lines = resMap.get(shell);
			for (Integer line : lines) {
				if (line <= 0) {
					continue;
				}
				inst.touch(line);
			}
			driver.addInstrumentation(shell, inst);
		}
		driver.generate(reportDir);
	}

	public static void main(String[] args) {
		try {
			File sourceDir = null;
			File reportDir = null;

			String var;
			var = System.getProperty("jp.co.sds_corp.tool.shell.sourceDir");
			if (var != null) {
				sourceDir = new File(var);
			}
			var = System.getProperty("jp.co.sds_corp.tool.shell.reportDir");
			if (var == null) {
				reportDir = new File("./report");
			} else {
				reportDir = new File(var);
			}
			for (String arg : args) {
				ShellCoverageParser parser = new ShellCoverageParser(arg);
				parser.generate(reportDir);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}
