package jp.co.sds_corp.tool.shell;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import com.jcoverage.coverage.Instrumentation;
import com.jcoverage.coverage.reporting.Main;
import com.jcoverage.coverage.reporting.collation.JavaFileLine;
import com.jcoverage.coverage.reporting.collation.JavaFilePage;
import com.jcoverage.coverage.reporting.collation.PackageSummaryPage;
import com.jcoverage.coverage.reporting.collation.ReportImpl;
import com.jcoverage.coverage.reporting.collation.ReportSummaryPackageLine;
import com.jcoverage.coverage.reporting.collation.ReportSummaryPage;
import com.jcoverage.coverage.reporting.collation.StaticFileCollator;
import com.jcoverage.coverage.reporting.html.MultiViewStaticHtmlFormat;
import com.jcoverage.reporting.Collator;
import com.jcoverage.reporting.FileSerializer;
import com.jcoverage.reporting.Format;
import com.jcoverage.reporting.Line;
import com.jcoverage.reporting.Page;
import com.jcoverage.reporting.Report;
import com.jcoverage.reporting.Serializer;

public class ShellReportDriver
{
	File javaSourceDirectory;
	Report report = new ReportImpl();
	Page indexPage;

	public ShellReportDriver(File javaSourceDirectory) {
		this.javaSourceDirectory = javaSourceDirectory;
		this.indexPage = this.report.createFrontPage();
	}

	public synchronized void addInstrumentation(String clzName, Instrumentation instrumentation) {
		String sourcePath = clzName;
		String packageName = "default";

		Line packageLine = this.indexPage.lookupLineByField(ReportSummaryPage.CATEGORY_PACKAGE_SUMMARY, ReportSummaryPackageLine.COLUMN_PACKAGE_NAME, packageName);
		Page packageDetailPage = null;
		if (packageLine == null)
		{
			packageLine = this.indexPage.createLine(ReportSummaryPage.CATEGORY_PACKAGE_SUMMARY);
			packageLine.setField(ReportSummaryPackageLine.COLUMN_PACKAGE_NAME, packageName);
			packageDetailPage = packageLine.openDetailPage();
		}
		else {
			packageDetailPage = packageLine.getDetailPage();
		}

		Line javaFileLine = packageDetailPage.lookupLineByField(PackageSummaryPage.CATEGORY_JAVAFILES, JavaFileLine.COLUMN_FILE_NAME, clzName);
		Page javaFileDetailPage = null;
		if (javaFileLine == null)
		{
			javaFileLine = packageDetailPage.createLine(PackageSummaryPage.CATEGORY_JAVAFILES);
			String filename = clzName;
			if (filename.startsWith("/")) {
				filename = "." + clzName;
			}
			javaFileLine.setField(JavaFileLine.COLUMN_FILE_NAME, filename);
			javaFileLine.setField(JavaFileLine.COLUMN_PATH, new File(this.javaSourceDirectory, sourcePath).getAbsolutePath());
			javaFileDetailPage = javaFileLine.openDetailPage();
		}
		else {
			javaFileDetailPage = javaFileLine.getDetailPage();
		}

		this.indexPage.addLineReference(javaFileLine, PackageSummaryPage.CATEGORY_JAVAFILES);
		((JavaFilePage)javaFileDetailPage).addInstrumentation(instrumentation);
	}

	public void generate(File outputDir) throws Exception {
		File imagesDir = new File(outputDir, "images");
	    imagesDir.mkdirs();
	    copyResource("red.gif", imagesDir);
	    copyResource("green.gif", imagesDir);

	    Collator collator = new StaticFileCollator(".html");
		this.report.setCollator(collator);
		Format htmlFormat = new MultiViewStaticHtmlFormat();
		Serializer serializer = new FileSerializer(outputDir);
		collator.addOutputter(htmlFormat, serializer);
		this.indexPage.close();
	}

	  static byte[] buf = new byte[1024];
	  static void copyResource(String resname, File dir)    throws IOException
	  {
	    FileOutputStream fos = new FileOutputStream(new File(dir, resname));
	    InputStream in = new BufferedInputStream(Main.class.getResourceAsStream(resname));
	    while (true) {
	      int n = in.read(buf, 0, buf.length);
	      if (n == -1) {
	        break;
	      }

	      fos.write(buf, 0, n);
	    }
	    in.close();
	    fos.close();
	  }
}

